#!/bin/bash -u

# Example script calls to produce masks for the CARS
# image W4m0m0_i.V1.7A.swarp.cut.fits and the associated 
# weight W4m0m0_i.V1.7A.swarp.cut.weight.fits
#
# NOTE:
# The routines used in this script were specifically 
# developed for MEGAPRIME@CFHT images.Parameters are tuned
# for 'empty field' observations such as CARS or CFHTLS;
# see the publication Erben et al. 2009 (A&A 493, 1197).
# For a documentation of the script you need to study their
# source code :-) I do not make a bigger effort to describe
# the internals here.
#
# For scripts that were developed for a more general context
# see the corresponding 'demo_WFI.sh' example.

# NECESSARY SETUP TO RUN THIS SCRIPT:
# - Install the THELI pipeline 'theli-X.XX.XX.tgz'
# - Install the automask package 'automask-X.XX.XX.tar.gz'
# - create 'somewhere' a working directory, say 'masking'
#   (simply <MASKING> henceforth)
#   - Copy the file 'progs.ini' from ../theli-X.XX.XX/scripts/Linux
#     (or .../theli-X.XX.XX/scripts/Linux_64; it depends on 
#      your machine which one you have)
#     to <MASKING>
#   - Copy ALL files from .../automask/Linux(_64) to <MASKING>
# - Install the GSC catalogue 'GSC.tar.gz'
#   - After unpacking go to .../GSC/src/ and see whether the program
#     'gsc.exe' works; if not you have to compile it with the Makefile
#     in this directory; sorry I did not write down whether there were
#     problems when I did this. You have to make sure that 'gsc.exe'
#     finally ends up in .../GSC/bin !

# To call this script you need to copy it to <MASKING>.
# I assume that automask scripts and progs.ini are now in the current
# directory; all images are in the dir. pointed to by the IMAGEDIR
# var. below. The GSC catalogue be in GSCDIR (see below)

IMAGEDIR=../images
GSCDIR=../GSC
AUTOMASK=../automask # where is 'automask' installed

# needed for the GSC1 utility program retrieving object data:
export GSCDAT=${GSCDIR}

# needed for the script calls below; a file with name 
# ${INSTRUMENT}_mask.ini within <MASKING> needs to be read:
export INSTRUMENT=MEGAPRIME

# The image and weight to be analysed
ANALYSEIMA=W4m0m0_i.V1.7A.swarp.cut.fits
ANALYSEWEIGHT=W4m0m0_i.V1.7A.swarp.cut.weight.fits

# some sanity checks:
test -f ${IMAGEDIR}/${ANALYSEIMA} ||\
  { echo "Image ${IMAGEDIR}/${ANALYSEIMA} not present! Exiting!"; exit 1; }

test -f ${IMAGEDIR}/${ANALYSEWEIGHT} ||\
  { echo "Image ${IMAGEDIR}/${ANALYSEWEIGHT} not present! Exiting!"; exit 1; }

test -f ${GSCDIR}/bin/gsc.exe ||\
  { echo "File ${GSCDIR}/bin/gsc.exe not present! Exiting!"; exit 1; }

#
# start with main script work:
#
# To have a prefix name for created catalogues, masks etc.:
BASE=`basename ${ANALYSEIMA} .fits`

# In the following we create masks for stellar sources. We found that
# best results are obtained if using the GSC1 catalogue for bright and
# the GSC2 catalogue for faint stars. 
#
# The following call to 'maskstars.sh' creates masks for bright, and
# faint stars. In addition, an attempt is made to mask large haloes
# around very bright point sources: 
tput bold
echo "Creating masks for stars and haloes"
tput sgr0
# The main output from the following script call is the file
# 'W4m0m0_i.V1.7A.swarp.cut_maskstars.reg' which contains the complete
# stellar mask set. Subsets of this file are contained in:
#
# - W4m0m0_i.V1.7A.swarp.cut_maskstars.tight.reg:
#   stellar masks without haloes (green masks)
# - W4m0m0_i.V1.7A.swarp.cut_maskstars.wide.reg:
#   magenta haloe masks; these are supposed to be halo mask
#   that should be applied in any case (bright magnitude 
#   limit of mag=10.35).
# - W4m0m0_i.V1.7A.swarp.cut_maskstars.wide2.reg:
#   cyan stellar halo mask which might or might not be necessary
#   (magnitude limit of mag=11.0)
# - W4m0m0_i.V1.7A.swarp.cut_maskstars_tw.reg:
#   a combination of the tight and the wide mask files.
# - W4m0m0_i.V1.7A.swarp.cut_maskstars.reg:
#   a combination of the tight, the wide and the wide2 mask files.
./maskstars.sh ${IMAGEDIR} \
               ${ANALYSEIMA} ${ANALYSEWEIGHT}
 
# create 'object density' masks:
tput bold
echo "Creating masks for underdense and overdense regions"
tput sgr0
./maskvoids.sh -i ${IMAGEDIR}/${ANALYSEIMA} \
                -w ${IMAGEDIR}/${ANALYSEWEIGHT} \
                -r ${BASE}_voids_tmp.reg -d 4 -g 16

# make the voids regions appear in yellow (for better distinction):
awk '$0 ~ /POLYGON/ {print $0 " # color=yellow"}' ${BASE}_voids_tmp.reg >\
  ${BASE}_voids.reg

rm ${BASE}_voids_tmp.reg


# In the following we want to mask objects which contain pixels of
# weight zero and which have not been masked by the steps done
# hitherto; see also comments before the 'create_flaggedstars_masks.sh' 
# command; very probably you need to heavily modify this part of
# the masking if your data were not processed by THELI.
#
# First, paste the masks created hitherto; if masking zero-weight objects
# we want to exclude objects which have a mask already.
cat ${BASE}_maskstars.reg ${BASE}_voids.reg > ${BASE}_star_voids.reg

# create a flag map with all saturated (weight zero) pixels set to '1'; 
# all other pixels set to zero:
tput bold
echo "creating Flag map ${IMAGEDIR}/${BASE}.flag.fits"
tput sgr0
ic -p 8 '1 0 %1 fabs 1.0e-06 < ?' ${IMAGEDIR}/${ANALYSEWEIGHT} > \
  ${IMAGEDIR}/${BASE}.flag.fits

# Sometimes, there are saturated objects or objects having pixels
# with zero weight. We want to mask those as well!
# In THELI, objects affected by this have a zero weight. We create
# a flag which has a '1' where the weight image is zero and a '0'
# otherwise. The script 'create_flaggedstars_masks.sh' then masks
# objects which are flagged but which are not yet masked:
#
tput bold
echo "flagging remaining saturated/zero weight features"
tput sgr0
./create_flaggedstars_masks.sh ${IMAGEDIR}\
  ${ANALYSEIMA} ${ANALYSEWEIGHT} ${BASE}.flag.fits \
  MEGAPRIME \
  ./${BASE}_star_voids.reg \
  ./${BASE}_saturated.reg

# The linear co-addition process of THELI leaves short asteroid trails
# in the images. The following script call tries to catch these
# features. Note that this step is VERY time consuming.  You probably
# NEVER need it if you do not use a mean, but a median co-addition (of
# course you have lots of other difficulties then!).
tput bold
echo "masking asteroids"
tput sgr0

# Note that 'testcat' is a dummy argument in the following call.
./maskasteroids.sh ${IMAGEDIR}\
  ${ANALYSEIMA} ${ANALYSEWEIGHT}\
  testcat ./${BASE}_star_voids.reg

# The resulting file from the asteroid masking is
# ${BASE}_maskast.reg. There are lots of temporary
# files with similar names created.

# build up the final, global mask file:
FINALMASKS=""

if [ -f ./${BASE}_star_voids.reg ]; then
  FINALMASKS="${FINALMASKS} ./${BASE}_star_voids.reg"  
fi

if [ -f ./${BASE}_saturated.reg ]; then
  FINALMASKS="${FINALMASKS} ./${BASE}_saturated.reg"  
fi

if [ -f ./${BASE}_maskast.reg ]; then
  FINALMASKS="${FINALMASKS} ./${BASE}_maskast.reg"  
fi

# finally paste all obtained mask to a 'final' one:
cat ${FINALMASKS} > ${BASE}_final.reg
 
# clean up and bye
rm ./${BASE}_star_voids.reg

exit 0;
