#!/usr/bin/env bash

# 02.08.2007
# I changed the PATH modification in the
# maskstars script to ensure calling the
# 'xy2sky' program of the automask package.
#
# 28.08.2007
# Bug fix on a test which tested for the
# presence of a 'file' instead for the
# presence of a directory
#
# 19.11.2007:
# Documentation added
#
# 21.11.2007:
# - Documentation updated
# - Installation modes included
#
# 24.07.2008:
# script expanded to include Tim's stellar masking 
# routines (developed primarily for CFHT CARS data)
#
# 15.09.2008:
# upgrade to version 3.7.5 of the wcstools
#
# 24.02.2009:
# I included the 'create_flaggedstars_masks.sh' script!
# (update of MASKCONF variable)
#
# 02.10.2009:
# - I updated script documentation
# - I included the 'convert_mask.sh' script!

# Usage information:
function printUsage
{
  echo "SCRIPT NAME:"
  echo "   install.sh - installs the automask package"
  echo ""
  echo "SYNOPSIS:"
  echo "   install.sh -m [ -f, -h ]"
  echo ""
  echo "OPTIONS:"
  echo "     -m MODE    # MODE of installation:"
  echo "                # ALL: C-sources are compiled and masking "
  echo "                #      scripts are created"
  echo "                # COMPILE: C-sources are compiled"
  echo "                # UPDATE: masking scripts are created"
  echo "                  "
  echo "   [ -f         # force complete reinstallation; also if modules ]     "
  echo "   [            # have been compiled previously                  ]"
  echo "   [ -h         # show this help and leave                       ]     "
  echo ""
  echo ""
  echo "DESCRIPTION:"
  echo "   The script installs the 'automask' package on your machine.  Except"
  echo "   the automask sources and scripts it consists of several source"
  echo "   modules including the external source packages 'cdsclient'"
  echo "   (Francois Ochsenbeins CDS client software v2.87), 'CFITSIO' (v3.03)"
  echo "   and 'wcstools' (v3.7.5).  The tar archive contains source codes for"
  echo "   all those packages and the 'install' script automatically compiles"
  echo "   them."
  echo ""
  echo "   After succesfully running this script (install mode 'COMPILE' or 'ALL')"
  echo "   you find a subdirectory"
  echo "   .../automask/bin/\${MACHINE} with the binary 'maskvoid'"
  echo "   executable. The external programs and library files are in"
  echo "   .../automask/ext_libs/\${MACHINE}."
  echo "   Put .../automask/ext_libs/\${MACHINE} into your PATH variable so that"
  echo "   the executables therein are found by your system."
  echo ""
  echo "   Currently, we support the MACHINES:"
  echo "   - Cygwin:   Cygwin on Windows computers"
  echo "   - Darwin:   Darwin on Power PPC and Intel Macs"
  echo "   - Linux:    32-bit Linux machines"
  echo "   - Linux_64: 64-bit Linux machines"
  echo "   - SunOS:    SUN machines"
  echo ""
  echo "   If you need to compile for another machine, add a config.  file in"
  echo "   the .../automask/ext_libs/wcstools-3.6.8 and .../automask/src"
  echo "   directories; MACHINE is determined via the 'uname' command."
  echo ""
  echo "   The main masking scripts 'maskvoids.sh' and 'maskstars_simple.sh'"
  echo "   are put into .../automask/scripts/\${MACHINE} (install mode ALL or"
  echo "   UPDATE) If you want to run them in another directory you need to"
  echo "   copy the 'sexconfig_include' file together with the scripts."
  echo "    "
  echo "   This install script can be run without modifications on different"
  echo "   UNIX architectues. The corresponding libraries and executables are"
  echo "   always put in approporiate \${MACHINE} subdirectories."
  echo ""
  echo "REQUIREMENTS:"
  echo "   To use the automask package, you need the THELI pipeline installed"
  echo "   on your system; you need THELI version 0.12.14 or higher."
  echo "   "
  echo "AUTHOR:"
  echo "   Thomas Erben (terben@astro.uni-bonn.de)"
}


# read command line arguments.

FORCE=0      # Force recompile of all sources;
             # also if certain libraries are
             # already present.

MODE=""

GoOn=0
while [ $GoOn = 0 ]
do
  case $1 in
  -f)       
      FORCE=1
      shift
      ;;
  -h)       
      printUsage
      exit 0;	   
      ;;
  -m)       
      MODE=$2
      shift 2
      ;;
   *)
      GoOn=1
      ;;
  esac
done

if [ -z ${MODE} ]; then
  printUsage
  exit 1;
fi

#
# test the machine type
#
function testmachine() {
  # first test if the endian program
  # is alreay available; if not, compile
  # it!
  if [ ! -f endian.c ]; then
    echo "need the source file 'endian.c'"
    exit 1
  else
    test -f endian && rm endian
    gcc -o endian endian.c	 
  fi

  # test for Cygwin machine; on my Windows/Cygwin 
  # computer the command 'uname' gives 'CYGWIN_NT-5.1'.
  # I assume that this is not the same for all systems,
  # hence the following, separate test for this system.
  # Note that 'uname -o' gives GNU/Linux for Linux systems
  # whereas simply 'uname' gives the wanted 'Linux'. Hence,
  # we do not generally use 'uname -o'.
  TESTCYGWIN=`uname -o`

  if [ "${TESTCYGWIN}" = "Cygwin" ]; then
    MACHINE="Cygwin"
  else
    MACHINE=`uname`
    #
    # test for Linux 64 architecture if
    # the machine is Linux
    if [ "${MACHINE}" = "Linux" ]; then
      LONG=`./endian | awk '($1=="Sizes:") {print substr($5,3,1)}'`
  
      if [ "${LONG}" = "8" ]; then
        MACHINE="Linux_64"
      fi
    fi
  fi

  export MACHINE
}

testmachine

DIR=`pwd`

if [ "${MODE}" = "ALL" ] || [ "${MODE}" = "COMPILE" ]
then 
  # test whether 'cfitsio' is already installed. If not,
  # install it:
  
  if [ ! -f ./ext_libs/${MACHINE}/lib/libcfitsio.a ] || \
     [ "${FORCE}" = "1" ]
  then
    cd ./ext_libs/cfitsio
    # we disable the creation of shared library as we
    # do not need them
    ./configure --prefix=${DIR}/ext_libs/${MACHINE} -disable-shared
    make
    make install
    make clean
  else
    echo "CFITSIO already present! Skipping its installation."
  fi
  
  cd ${DIR}
  
  # test whether 'cdsclient' is already installed. If not,
  # install it:
  
  if [ ! -f ./ext_libs/${MACHINE}/bin/aclient ] || \
     [ "${FORCE}" = "1" ]
  then
    cd ./ext_libs/cdsclient-2.87
    ./configure --prefix=${DIR}/ext_libs/${MACHINE}
    make
    make install
    make clean
  else
    echo "cdsclient already present! Skipping its installation."
  fi
  
  cd ${DIR}
  
  # test whether 'wcstools' is already installed. If not,
  # install it:
  
  if [ ! -f ./ext_libs/${MACHINE}/bin/sky2xy ] || \
     [ "${FORCE}" = "1" ]
  then
    cd ./ext_libs/wcstools-3.7.5
    make
    make clean
  else
    echo "cdsclient already present! Skipping its installation."
  fi
  
  cd ${DIR}
  
  cd ./src
  make
  test -d ${DIR}/bin/${MACHINE} || mkdir -p ${DIR}/bin/${MACHINE}
  cp maskvoids ${DIR}/bin/${MACHINE}
  make clean
  
  cd ${DIR}
fi

if [ "${MODE}" = "ALL" ] || [ "${MODE}" = "UPDATE" ]
then 
  # update scripts
  test -d ./scripts/${MACHINE} || mkdir ./scripts/${MACHINE}
  
  sed -e 's!BINPATH=!BINPATH='${DIR}'/bin/'${MACHINE}'!' \
      ./scripts/maskvoids.sh.raw \
      > ./scripts/${MACHINE}/maskvoids.sh
  
  sed -e 's!BINPATH=!BINPATH='${DIR}'/ext_libs/'${MACHINE}'/bin!' \
      ./scripts/maskstars_simple.sh.raw \
      > ./scripts/${MACHINE}/maskstars_simple.sh
  
  sed -e 's!BINPATH=!BINPATH='${DIR}'/ext_libs/'${MACHINE}'/bin!' \
      ./scripts/maskstars.sh.raw \
      > ./scripts/${MACHINE}/maskstars.sh
  
  sed -e 's!MASKCONF=!MASKCONF='${DIR}'/config/!' \
      ./scripts/create_flaggedstars_masks.sh.raw \
      > ./scripts/${MACHINE}/create_flaggedstars_masks.sh
  
  sed -e 's!MASKCONF=!MASKCONF='${DIR}'/config/!' \
      ./config/MEGAPRIME_mask.ini.raw \
      > ./scripts/${MACHINE}/MEGAPRIME_mask.ini
  
  cp ./scripts/sexconfig_include ./scripts/${MACHINE}
  cp ./scripts/getgsc.sh.raw ./scripts/${MACHINE}/getgsc.sh
  cp ./scripts/maskasteroids.sh.raw ./scripts/${MACHINE}/maskasteroids.sh
  cp ./scripts/convert_mask.sh.raw ./scripts/${MACHINE}/convert_mask.sh

  cp ./scripts/*pl ./scripts/${MACHINE}
  
  chmod +x ./scripts/${MACHINE}/*sh
  chmod +x ./scripts/${MACHINE}/*pl
fi
